# SimDedup

SimDedup includes the following two parts: 
  - DAN (Deduplication ANalyzer) and
  - SIMD (high-fidelity reliability Simulator IMproved for Deduplication). 

## Overview
### DAN
DAN (Deduplication ANalyzer) is used to parse and collect information of chunks and files in production traces (i.e., [FSL](http://tracer.filesystems.org) and [MS](http://iotta.snia.org/tracetypes/6)). Also it analyzes deduplication characteristics of production traces. Finally, the collected data of traces serve to calculate data loss for the deduplication model in SIMD.

### SIMD
SIMD is a discrete-event simulator that characterizes the reliability of a disk-based RAID with and without deduplication. It takes a file system snapshot or an aggregate of multiple file system snapshots, failure and repair distributions, and a system mission time (e.g., 10 years) as inputs. Based on NOmarlized Magnitude of Data Loss (NOMDL), it can report four reliability metrics: 

- expected number of corrupted chunks per TB, 
- expected number of corrupted files per TB,
- expected size (in bytes) of corrupted chunks per TB, and 
- expected size (in bytes) of corrupted files per TB.

## Prerequisites

### DAN
1. Please install `libglib2.0-dev` by 

   ```
   apt-get install libglib2.0-dev
   ```

2. Make `/usr/lib/libglib.so` a symbolic link to `libglib-2.0.so` by

   ```
   ln -sf /usr/lib/x86_64-linux-gnu/libglib-2.0.so /usr/lib/libglib.so
   ```

3. Move `glibconfig.h` to the same directory of `glib.h`.

4. Move all headers in `glib-2.0/` to `/usr/include/`.

5. Install Redis Database.

### SIMD
Please install [mpmath](http://code.google.com/p/mpmath/), [numpy](http://www.numpy.org/) and [scipy](http://www.scipy.org/).

## Usage

  - DAN

    Please refer to [README](dan/README.md) in `dan/`.

  - SIMD

    Please refer to [README](simd/README.md) in `simd/`.

## Contact
Please email to Shujie Han (sjhan@cse.cuhk.edu.hk) if you have any questions.

## Publications

  - Min Fu, [Patrick P. C. Lee]((http://www.cse.cuhk.edu.hk/~pclee)), Dan Feng, Zuoning Chen, Yu Xiao
    "A Simulation Analysis of Reliability in Primary Storage Deduplication", *IISWC 2016*.
