# DAN

DAN (Deduplication ANalyzer) is used to parse and collect information of chunks and files in production traces (i.e., [FSL](http://tracer.filesystems.org) and [MS](http://iotta.snia.org/tracetypes/6)). Also it analyzes deduplication characteristics of production traces. Finally, the collected data of traces serves to calculate data loss for the deduplication model in SIMD.

## Table of Contents
  - [Prerequisites](#prerequisites)
  - [Overview](#overview)
    - [Collector](#collector)
    - [Analyzer](#analyzer)
    - [Library](#library)
  - [Usage](#usage)
    - [Collector](#collector2)
    - [Analyzer](#analyzer2)

## Prerequisites <a name="prerequisites"></a>

1. Please install `libglib2.0-dev` by 

   ```
   apt-get install libglib2.0-dev
   ```

2. Make `/usr/lib/libglib.so` a symbolic link to `libglib-2.0.so` by

   ```
   ln -sf /usr/lib/x86_64-linux-gnu/libglib-2.0.so /usr/lib/libglib.so
   ```

3. Move `glibconfig.h` to the same directory of `glib.h`.

4. Move all headers in `glib-2.0/` to `/usr/include/`.

5. Install Redis Database.

## Overview <a name="overview"></a>

####  Collector <a name="collector"></a>

  It is used to collect the main information of chunks and files into Redis.

  - FSL
    `fsl/` is used for FSL datasets.
    - **collector** goes through the snapshots in FSL and build the chunk, file, region, and container databases in Redis.
    - **simd_exporter** simulates USEs and UDFs when exporting information of chunks and files with/without deduplication from Redis according to four metrics (i.e., chunk vs. file and weighted vs. non-weighted).
    - **simd_reverse_exporter** is similar to **simd_exporter** except doing logical repair in reverse order for UDFs.

  - MS
    `ms/` is used for MS datasets.
    - **collector**
      - It goes through the snapshots in MS with indicated deduplication domain size;
      - builds the chunk, file, region, and container databases in Redis.
    - **simd_exporter**
      - It requires to indicate deduplication domain size;
      - simulates USEs and UDFs, which is similar to **simd_exporter** in `fsl/`.

#### Analyzer <a name="analyzer"></a>

  It is used to analyze redundancy sources. Please refer to Section 4 in our paper.
  - FSL
    `fsl/` is used for FSL datasets.
    - **refs_distrbution_exporter** prints reference count of each chunk.
    - **file_refs_source_analyzer** reports the percentages of each redundancy sources.
    - **file_exporter** exports files according to different redundancy source.

  - MS
    `ms/` is used for MS datasets.
    - **refs_distrbution_exporter** prints reference count of each chunk.
    - **file_refs_source_analyzer** reports the percentages of each redundancy sources.
    - **file_exporter** exports files according to different redundancy sources.

  - common
    Based on the results of **file_exporter**, it analyzes file sizes and types from different redundancy sources 
    - **file_intra_check.py**
      - It checks _Intra_ and reports file sizes and types.
    - **file_dup_check.py**
      - It checks _DupFile_ and reports file names, sizes, and types.
    - **file_min_check.py**
      - It checks _Min_ and reports file names, sizes, and types.
    - **file_missed_check.py**
      - It checks _Missed_ and reports file size and coefficient.

#### Library <a name="library"></a>

  It is used to parse FSL and MS datasets and store information of chunks and files in Redis.

## Usage <a name="usage"></a>

#### Collector <a name="collector2"></a>
  - FSL
    - **collector**
      `./collector [FSL path]`
    - **simd_exporter**
      ```
      ./simd_exporter [options] [path]
        options:
          default = chunk level, no deduplication, non-weighted, no pophash_file
          -f = file level
          -d = deduplication
          -w = weighted
          -p <pophash_file> = inject pophash_file of DCT
        path = path of FSL traces
      ```
    - **simd_reverse_exporter**
        The usage is similar to **simd_exporter**.
  - MS
    - **collector**
      ```
      ./collector [chunking_size(KB)] [chunking_method] [count] [dir] [files]
        chunking_size(KB) = 8, 16,...
        chunking_method = f (fixed), v (variable)
        count = deduplication domain size
        dir = directory path of Microsoft traces
        files = traces files (Microsoft snapshots)
      ```
      Example
      ```
      ./collector 8 v 1 ms_dir/ 0001
      ```
    - **simd_exporter**
      ```
      ./simd_exporter [options] [count] [dir] [files]
        options:
          default = chunk level, no deduplication, non-weighted, no pophash_file
          -f = file level
          -d = deduplication
          -w = weighted
          -p <pophash_file> = inject pophash_file of DCT
        count = deduplication domain size
        dir = directory path of Microsoft traces
        files = traces files (Microsoft snapshots)
      ```
      Example
      ```
      ./simd_exporter 1 ms_dir/ 0001
      ```
#### Analyzer <a name="analyzer2"></a>
  - FSL & MS
    - **refs_distrbution_exporter**
      ```
      ./refs_distribution_exporter [options]
        options:
          default = get reference count per chunk
          -p <popularity> = get the chunks with referecent count more than popularity
      ```
      Example
        Run `./refs_distribution_exporter -p 5` and get the chunks with referenced count more than 5
    - **file_refs_source_analyzer**
      Run `./file_refs_source_analyzer` to get percentages of each redundancy source.
    - **file_exporter**
      ```
      ./file_exporter [options]
        options:
          -i = Intra
          -d = DupFile
          -m = Min
          -s = Missed
      ```
      Example
        Run `./file_exporter -i` and get the files with intra-file redundancy.
  - common
    - **file_intra_check.py**
      ```
      python file_intra_check.py [options]
      -p = get popular types
      -s = get popular file sizes
      ```
    - **file_dup_check.py**
      ```
      python file_dup_check.py [options]
      -n = check names and types
      -t = get popular types
      -s = get popular file sizes
      -r = get file references distribution
      ```
    - **file_min_check.py**
      ```
      python file_min_check.py [options]
      -n = check names and types
      -t = get popular types
      -s = get popular file sizes
      ```
    - **file_missed_check.py**
      ```
      python file_missed_check.py [options]
      -f = print the missed file set
      -s = get file sizes distribution
      -c = check each pair and reports coefficient
      ```
