#include "../../lib/ms/store.h"
#include <assert.h>
#include <getopt.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void get_file2ref_ratio(unsigned int lb, unsigned int rb) {
  init_iterator("CHUNK");

  struct chunk_rec r;
  memset(&r, 0, sizeof(r));

  float sum = 0;
  int count = 0;

  while (iterate_chunk(&r, 0) == 0) {
    if (r.rcount >= lb && r.rcount <= rb) {
      fprintf(stdout, "%10.5f\n", 1.0 * r.fcount / r.rcount);
      sum += 1.0 * r.fcount / r.rcount;
      count++;
    }
  }

  fprintf(stderr, "avg: %10.5f\n", sum / count);

  close_iterator();
}

void analyze_references_source(unsigned int lb, unsigned int rb) {
  init_iterator("CHUNK");

  struct chunk_rec r;
  memset(&r, 0, sizeof(r));

  int k = 0;
  int total_references = 0;
  int intra_file = 0;
  int identical_file = 0;
  int min_similar_file = 0;
  int similar_file = 0;
  int same_suffix_file = 0;
  int distinct_file = 0;
  while (iterate_chunk(&r, 1) == 0) {
    if (r.rcount >= lb && r.rcount <= rb) {
      total_references += r.rcount - 1;
      intra_file += r.rcount - r.fcount;
      if (r.fcount == 1)
        continue;

      fprintf(stderr, "fcount = %d\n", r.fcount);
      struct file_rec *files; // files[r.fcount]
      files = (struct file_rec *)malloc((r.fcount) * sizeof(struct file_rec));
      if (files == NULL) {
        fprintf(stderr, "cannot allocate memory!\n");
        exit(1);
      }
      for (k = 0; k < r.fcount; k++) {
        files[k].fid = 0;
        files[k].fname = 0;
        files[k].cnum = 0;
        files[k].fsize = 0;
        files[k].hash[20] = "";
        files[k].minhash[20] = "";
        files[k].maxhash[20] = "";
        files[k].fextension[10] = "";
      }
      // memset(files, 0, (size_t)r.fcount*sizeof(struct file_rec));
      int i = 0;
      for (; i < r.fcount; i++) {
        files[i].fid = r.list[r.rcount + i];
        search_file(&files[i]);
      }

      /* analyze files */
      for (i = 0; i < r.fcount - 1; i++) {
        int j = i + 1;
        int identical = 0, min_similar = 0, similar = 0, same_suffix = 0;
        for (; j < r.fcount; j++) {
          if (memcmp(files[i].hash, files[j].hash, sizeof(files[i].hash)) ==
              0) {
            identical = 1;
            break;
          } else if (memcmp(files[i].minhash, files[j].minhash,
                            sizeof(files[i].minhash)) == 0) {
            min_similar = 1;
          } else if (memcmp(files[i].maxhash, files[j].maxhash,
                            sizeof(files[i].maxhash)) == 0) {
            similar = 1;
          } else if (strcmp(files[i].fextension, files[j].fextension) == 0) {
            fprintf(stderr, "fname %s and %s\n", files[i].fname,
                    files[j].fname);
            fprintf(stderr, "extension %s and %s\n", files[i].fextension,
                    files[j].fextension);
            same_suffix = 1;
          }
        }
        if (identical == 1)
          identical_file++;
        else if (min_similar == 1)
          min_similar_file++;
        else if (similar == 1)
          similar_file++;
        else if (same_suffix == 1)
          same_suffix_file++;
        else
          distinct_file++;
      }
      free(files);
    }
  }
  fprintf(stderr, "here\n");
  fprintf(stderr, "Intra: %.5f\n", 1.0 * intra_file / total_references);

  close_iterator();
  assert(total_references == intra_file + identical_file + min_similar_file +
                                 similar_file + same_suffix_file +
                                 distinct_file);

  if (total_references == 0) {
    fprintf(stderr, "total_references=0\n");
  } else {
    fprintf(stderr, "%8s %8s %8s %8s %8s %8s\n", "Intra", "DupFile", "Min",
            "+Max", "Suffix", "Missed");
    fprintf(stdout, "%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
            1.0 * intra_file / total_references,
            1.0 * identical_file / total_references,
            1.0 * min_similar_file / total_references,
            1.0 * similar_file / total_references,
            1.0 * same_suffix_file / total_references,
            1.0 * distinct_file / total_references);
  }
}

int main(int argc, char *argv[]) {
  int opt = 0;
  unsigned int lb = 2, rb = -1;
  int task = 0;
  while ((opt = getopt_long(argc, argv, "l:r:t", NULL, NULL)) != -1) {
    switch (opt) {
    case 'l':
      lb = atoi(optarg);
      break;
    case 'r':
      rb = atoi(optarg);
      break;
    case 't':
      task = 1;
      break;
    default:
      return -1;
    }
  }

  /* makes no sense to analyze reference number 1 */
  assert(lb > 1);
  open_database();

  if (task == 1)
    get_file2ref_ratio(lb, rb);
  else
    analyze_references_source(lb, rb);

  close_database();

  return 0;
}
