#include "../../lib/ms/store.h"
#include <assert.h>
#include <getopt.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/fcntl.h>

static void print_hash(const uint8_t *hash, int hash_size_in_bytes) {
  int j;
  printf("%.2hhx", hash[0]);
  for (j = 1; j < hash_size_in_bytes; j++)
    printf(":%.2hhx", hash[j]);
  printf("\n");
}

void get_reference_per_chunk() {
  init_iterator("CHUNK");

  int max = 1;
  int csize = 0;
  char maxhash[20];

  int count = 0;
  /* stat[10] for >10 */
  /* stat[11] for >20 */
  int stat[32];
  memset(stat, 0, sizeof(stat));

  struct chunk_rec r;
  memset(&r, 0, sizeof(r));
  while (iterate_chunk(&r, 0) == 0) {
    // fprintf(stdout, "%d\n", r.rcount);
    count++;
    if (r.rcount > max) {
      max = r.rcount;
      csize = r.csize;
      memcpy(maxhash, r.hash, 20);
    }
    if (r.rcount > 10000) {
      stat[31]++;
    } else if (r.rcount > 1000) {
      stat[30]++;
    } else if (r.rcount > 500) {
      stat[29]++;
    } else if (r.rcount > 100) {
      stat[28]++;
    } else if (r.rcount > 20) {
      stat[18 + r.rcount / 10]++;
    } else
      stat[r.rcount - 1]++;
  }

  fprintf(stderr, "max = %d, size = %d\n", max, csize);
  fprintf(stdout, "max = %d, size = %d\n", max, csize);
  fprintf(stderr, "hash = %.2hhx:%.2hhx:%.2hhx:%.2hhx:%.2hhx:%.2hhx\n",
          maxhash[0], maxhash[1], maxhash[2], maxhash[3], maxhash[4],
          maxhash[5]);
  int i = 0;
  for (; i < 20; i++) {
    fprintf(stderr, "[%2d : %10.10f]\n", i + 1, 1.0 * stat[i] / count);
    printf("[%2d : %10.10f]\n", i + 1, 1.0 * stat[i] / count);
  }
  int start = 20;
  int end = 30;
  for (i = 20; i < 28; i++) {
    fprintf(stderr, "[(%2d, %2d] : %10.10f]\n", start, end,
            1.0 * stat[i] / count);
    printf("[(%2d, %2d] : %10.10f]\n", start, end, 1.0 * stat[i] / count);
    start = end;
    end += 10;
  }
  fprintf(stderr, "[(100, 500] : %10.10f]\n", 1.0 * stat[28] / count);
  printf("[(100, 500] : %10.10f]\n", 1.0 * stat[28] / count);
  fprintf(stderr, "[(500, 1000] : %10.10f]\n", 1.0 * stat[29] / count);
  printf("[(500, 1000] : %10.10f]\n", 1.0 * stat[29] / count);
  fprintf(stderr, "[(1000, 10000] : %10.10f]\n", 1.0 * stat[30] / count);
  printf("[(1000, 10000] : %10.10f]\n", 1.0 * stat[30] / count);
  fprintf(stderr, "[(10000, inf] : %10.10f]\n", 1.0 * stat[31] / count);
  printf("[(10000, inf] : %10.10f]\n", 1.0 * stat[31] / count);

  close_iterator();
}

int get_popular_h() {
  init_iterator("CHUNK");

  int count = 0;
  int h = 1;

  int64_t logical_size = 0;
  int64_t physical_size = 0;

  int64_t pop_physical_size[21];
  int64_t pop_logical_size[21];
  struct chunk_rec r;
  memset(&r, 0, sizeof(r));
  /* pop_physical_size[20] for >20 */
  memset(pop_physical_size, 0, sizeof(pop_physical_size));
  memset(pop_logical_size, 0, sizeof(pop_logical_size));
  while (iterate_chunk(&r, 0) == 0) {
    // fprintf(stdout, "%d\n", r.rcount);
    count++;
    physical_size += r.csize;

    int64_t ls = r.csize;
    ls *= r.rcount;
    logical_size += ls;

    if (r.rcount > 20) {
      pop_physical_size[20] += r.csize;
      pop_logical_size[20] += ls;
    } else {
      pop_physical_size[r.rcount - 1] += r.csize;
      pop_logical_size[r.rcount - 1] += ls;
    }
  }
  // fprintf(stderr, "physical_size = %"PRId64"\n", physical_size);
  int i = 0;
  float pop_ratio[21];
  memset(pop_ratio, 0, sizeof(pop_ratio));
  for (; i < 20; i++) {
    pop_ratio[i] = 1.0 * pop_physical_size[i] / physical_size;
    fprintf(stderr, "[%2d : %10.5f]\n", i + 1, pop_ratio[i]);
    printf("[%2d : %10.5f]\n", i + 1, pop_ratio[i]);
  }
  pop_ratio[20] = 1.0 * pop_physical_size[20] / physical_size;
  fprintf(stderr, "[>20 : %10.5f]\n", pop_ratio[20]);
  printf("[>20 : %10.5f]\n", pop_ratio[20]);

  close_iterator();
  float tmp_sum = 0;
  for (i = 20; i >= 1; i--) { // make sure h >= 1 and r.rcount > h
    tmp_sum += pop_ratio[i];
    if (tmp_sum >= 0.01) {
      h = i;
      break;
    }
  }
  if (h == 0) {
    h++;
  }
  return h;
}
void output_popular_chunks(int h, char *filename) {
  init_iterator("CHUNK");

  int fd = open(filename, O_CREAT | O_WRONLY, S_IWUSR | S_IRUSR);

  int count = 0;
  int pop_count = 0;

  int64_t logical_size = 0;
  int64_t physical_size = 0;

  int64_t pop_physical_size = 0;
  int64_t pop_logical_size = 0;

  struct chunk_rec r;
  memset(&r, 0, sizeof(r));
  while (iterate_chunk(&r, 0) == 0) {
    count++;

    physical_size += r.csize;

    int64_t ls = r.csize;
    ls *= r.rcount;
    logical_size += ls;

    if (r.rcount > h) {
      pop_count++;

      pop_physical_size += r.csize;

      pop_logical_size += ls;

      // print_hash(r.hash, 20);

      write(fd, r.hash, 20);
    }
  }

  // fprintf(stderr, "physical_size = %"PRId64"\n", physical_size);
  fprintf(stderr, "percentage in count: %.6f\n", 1.0 * pop_count / count);
  fprintf(stderr, "percentage in physical capacity: %.6f\n",
          1.0 * pop_physical_size / physical_size);
  fprintf(stderr, "percentage in logical capacity: %.6f\n",
          1.0 * pop_logical_size / logical_size);
  close_iterator();

  close(fd);
}

const char *const short_options = "po:";
struct option long_options[] = {
    {"popular", 0, NULL, 'p'}, {"output", 0, NULL, 'o'}, {NULL, 0, NULL, 0}};

int main(int argc, char *argv[]) {
  int opt = 0;
  int popular_chunks = 0;
  int h = 10;
  char *filename = NULL;
  while ((opt = getopt_long(argc, argv, short_options, long_options, NULL)) !=
         -1) {
    switch (opt) {
    case 'p':
      popular_chunks = 1;
      // h = atoi(optarg);
      break;
    case 'o':
      filename = optarg;
      break;
    default:
      return -1;
    }
  }

  open_database();

  if (popular_chunks == 0) {
    get_reference_per_chunk();
  } else {
    h = get_popular_h();
    fprintf(stderr, "get popular h = %d\n", h);
    printf("get popular h = %d\n", h);
    output_popular_chunks(h, filename);
  }

  close_database();

  return 0;
}
