# SIMD

SIMD is a discrete-event simulator that characterizes the reliability of a disk-based RAID with and without deduplication. It takes a file system snapshot or an aggregate of multiple file system snapshots, failure and repair distributions, and a system mission time (e.g., 10 years) as inputs. Based on NOmarlized Magnitude of Data Loss (NOMDL), it can report four reliability metrics:

- expected number of corrupted chunks per TB,
- expected number of corrupted files per TB,
- expected size (in bytes) of corrupted chunks per TB, and
- expected size (in bytes) of corrupted files per TB.

We build SIMD on the High-Fidelity Reliability Simulator ([HFRS](http://www.kaymgee.com/Kevin_Greenan/Software_files/hfrs.tar)) developed by [Kevin Greenan](http://www.kaymgee.com/Kevin_Greenan/Welcome.html).

## Prerequisite

Please install [mpmath](http://code.google.com/p/mpmath/), [numpy](http://www.numpy.org/) and [scipy](http://www.scipy.org/).

## Usage

### Basics

- **Online mode**

  In this mode, if a failure event incurs any data loss, it will trigger a data loss event to  _deduplication model_, which performs sequential Monte Carlo simulation to calculate and output a set of reliability metrics based on the input file system snapshots.
  ```
  python simd.py
  -i <num_iterations> [--iterations <num_iterations>]
  -r <raid_type> [--raid <raid_type>]
  -t <trace> [--trace <trace_file>]
  -f [--filelevel]
  -d [--dedup]
  -w [--weighted]
  -o <output_file> [--output <output_file>]
  ```
  For more details, please run `python simd.py -h`.

- **Offline mode**

  In this mode, if a failure event incurs any data loss, it will output this event into a file, e.g., `events`, by running `python simd.py -o events`. Then running `python injector.py -e events` triggers the data loss event in `events` file to _deduplication model_ and calculate a set of reliability metrics based on the input file system snapshots.

  ```
  python injector.py
  -e <events_file>
  -f [--filelevel]
  -d [--dedup]
  -w [--weighted]
  -t <trace> [--trace <trace_file>]
  ```
